"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ffprobe = exports.exec = void 0;
const Path = require("path");
const fs_1 = require("fs");
const CP = require("child_process");
const util_1 = require("util");
exports.exec = (0, util_1.promisify)(CP.exec);
async function ffprobe(filePath, { path: ffprobePath = process.env.FFPROBE_PATH || 'ffprobe' } = {}) {
    filePath = Path.resolve(filePath);
    let rawData;
    let streams;
    try {
        const stat = await fs_1.promises.stat(filePath);
        const { stdout, stderr } = await (0, exports.exec)([
            `"${ffprobePath}"`,
            '-hide_banner',
            '-v error',
            '-show_streams',
            '-show_format',
            '-print_format',
            'json',
            `"${filePath}"`,
        ].join(' '));
        if (stderr)
            throw new Error(stderr);
        rawData = JSON.parse(stdout);
        if (!rawData || !Array.isArray(rawData.streams) || !rawData.format || typeof rawData.format !== 'object') {
            throw new Error(`Unsupported format. \n\nInvalid probe output: ${stdout}`);
        }
        rawData.format.size = stat.size;
        rawData.format.duration = (parseFloat(`${rawData.format.duration || 0}`) || 0) * 1000;
        streams = normalizeStreams(rawData);
    }
    catch (error) {
        throw new Error(`Unsupported format. Probing, parsing, or normalizing probed data failed: ${eem(error)}`);
    }
    let firstVideoStream;
    let firstAudioStream;
    let firstImageStream;
    let firstSubtitleStream;
    for (const stream of streams) {
        if (isVideoStream(stream) && !firstVideoStream)
            firstVideoStream = stream;
        if (isAudioStream(stream) && !firstAudioStream)
            firstAudioStream = stream;
        if (isImageStream(stream) && !firstImageStream)
            firstImageStream = stream;
        if (isSubtitlesStream(stream) && !firstSubtitleStream)
            firstSubtitleStream = stream;
    }
    const formatTags = lowercaseProps(removeNullProps(rawData.format.tags));
    if (firstVideoStream) {
        const duration = rawData.format.duration;
        if (!duration || duration <= 0) {
            throw new Error(`Unsupported format. Invalid format duration: ${JSON.stringify(rawData, null, 2)}`);
        }
        return {
            ...firstVideoStream.tags,
            ...formatTags,
            path: filePath,
            type: 'video',
            codec: firstVideoStream.codec,
            container: rawData.format.format_name,
            duration,
            framerate: firstVideoStream.framerate,
            width: firstVideoStream.width,
            height: firstVideoStream.height,
            sar: firstVideoStream.sar,
            dar: firstVideoStream.dar,
            displayWidth: firstVideoStream.displayWidth,
            displayHeight: firstVideoStream.displayHeight,
            size: rawData.format.size,
            streams,
            videoStreams: streams.filter(isVideoStream),
            audioStreams: streams.filter(isAudioStream),
            subtitlesStreams: streams.filter(isSubtitlesStream),
        };
    }
    if (firstAudioStream) {
        const duration = rawData.format.duration;
        if (!duration || duration <= 0) {
            throw new Error(`Unsupported format. Invalid format duration: ${JSON.stringify(rawData, null, 2)}`);
        }
        const cover = firstImageStream ? { ...firstImageStream } : undefined;
        if (cover)
            delete cover.disposition;
        return {
            ...firstAudioStream.tags,
            ...formatTags,
            path: filePath,
            type: 'audio',
            size: rawData.format.size,
            codec: firstAudioStream.codec,
            container: rawData.format.format_name,
            duration,
            channels: firstAudioStream.channels,
            language: firstAudioStream.language,
            cover,
        };
    }
    if (firstImageStream) {
        return {
            ...firstImageStream.tags,
            ...formatTags,
            path: filePath,
            type: 'image',
            size: rawData.format.size,
            codec: firstImageStream.codec,
            container: extractFileFormat(filePath),
            width: firstImageStream.width,
            height: firstImageStream.height,
            sar: firstImageStream.sar,
            dar: firstImageStream.dar,
            displayWidth: firstImageStream.displayWidth,
            displayHeight: firstImageStream.displayHeight,
        };
    }
    throw new Error(`Unknown file, unable to categorize probe data: ${JSON.stringify(rawData, null, 2)}`);
}
exports.ffprobe = ffprobe;
function normalizeStreams(rawData) {
    const rawStreams = rawData.streams;
    const seconds = rawData.format.duration / 1000;
    const streams = [];
    for (const rawStream of rawStreams) {
        const codec = normalizeCodecName(rawStream.codec_name);
        const tags = lowercaseProps(removeNullProps(rawStream.tags));
        const extractError = (what) => new Error(`Couldn't extract ${what} out of ${rawStream.codec_type} stream: ${JSON.stringify(rawStream, null, 2)}`);
        if (tags) {
            if (tags.comments && !tags.comment)
                tags.comment = tags.comments;
        }
        switch (rawStream.codec_type) {
            case 'subtitle': {
                streams.push({
                    ...rawStream,
                    type: 'subtitles',
                    codec,
                    disposition: rawStream.disposition,
                    tags,
                });
                break;
            }
            case 'audio': {
                const channels = rawStream.channels;
                if (channels == null)
                    throw extractError('channels');
                streams.push({
                    ...rawStream,
                    type: 'audio',
                    codec,
                    channels,
                    disposition: rawStream.disposition,
                    tags,
                });
                break;
            }
            case 'video': {
                const [frNum, frDen] = (rawStream.r_frame_rate || '').split('/').map((part) => parseFloat(part));
                const disposition = rawStream.disposition;
                let framerate = frNum && frDen ? frNum / frDen : false;
                const width = rawStream.width;
                const height = rawStream.height;
                if (typeof framerate !== 'number' || !Number.isFinite(framerate) || framerate <= 0) {
                    throw extractError('framerate');
                }
                else {
                    framerate = Math.round(framerate * 1e6) / 1e6;
                }
                if (!Number.isInteger(width) || width < 1)
                    throw extractError('width');
                if (!Number.isInteger(height) || height < 1)
                    throw extractError('height');
                const sar = parseAspectRatio(rawStream.sample_aspect_ratio) || 1;
                const dar = parseAspectRatio(rawStream.display_aspect_ratio) || (width / height) * sar;
                const displayWidth = sar > 1 ? Math.round(width * sar) : width;
                const displayHeight = sar > 1 ? height : Math.round(width / dar);
                if (!seconds ||
                    Math.abs(seconds - 1 / framerate) < 0.02 ||
                    disposition.attached_pic ||
                    disposition.timed_thumbnails) {
                    streams.push({
                        ...rawStream,
                        type: 'image',
                        codec,
                        width,
                        height,
                        sar,
                        dar,
                        displayWidth,
                        displayHeight,
                        disposition: rawStream.disposition,
                        tags,
                    });
                }
                else {
                    streams.push({
                        ...rawStream,
                        type: 'video',
                        codec,
                        width,
                        height,
                        sar,
                        dar,
                        displayWidth,
                        displayHeight,
                        framerate,
                        disposition: rawStream.disposition,
                        tags,
                    });
                }
                break;
            }
        }
    }
    return streams;
}
function normalizeCodecName(codecName) {
    const substitute = codecNameSubstitutes[codecName];
    return substitute ? substitute : codecName;
}
const codecNameSubstitutes = {
    mjpeg: 'jpeg',
};
function isVideoStream(value) {
    return value.type === 'video';
}
function isImageStream(value) {
    return value.type === 'image';
}
function isAudioStream(value) {
    return value.type === 'audio';
}
function isSubtitlesStream(value) {
    return value.type === 'subtitles';
}
function parseAspectRatio(value) {
    var _a;
    const groups = (_a = /^(?<numerator>\d+(\.\d+)?)((:|\/)(?<denominator>\d+(\.\d+)?))?$/.exec(`${value}`)) === null || _a === void 0 ? void 0 : _a.groups;
    if (!groups)
        return null;
    const numerator = groups.numerator ? parseInt(groups.numerator, 10) : undefined;
    const denominator = groups.denominator ? parseInt(groups.denominator, 10) : undefined;
    if (numerator == null || !Number.isFinite(numerator))
        return null;
    return denominator != null && Number.isFinite(denominator) ? numerator / denominator : numerator;
}
function eem(error, preferStack = false) {
    return error instanceof Error ? (preferStack ? error.stack || error.message : error.message) : `${error}`;
}
function lowercaseProps(obj) {
    const result = {};
    if (obj) {
        for (const [prop, value] of Object.entries(obj)) {
            result[prop.toLowerCase()] = value;
        }
    }
    return result;
}
function removeNullProps(obj) {
    const result = {};
    if (obj) {
        for (const [prop, value] of Object.entries(obj)) {
            if (value != null)
                result[prop] = value;
        }
    }
    return result;
}
function extractFileFormat(filename) {
    const ext = Path.extname(String(filename)).slice(1).toLowerCase();
    return extensionToFormat[ext] || ext;
}
const extensionToFormat = {
    jpeg: 'jpg',
    pjpeg: 'jpg',
};
